import urlparse
import bucho


default_message = """\
bucho provide below urls:
 * /show
 * /latest_status
 * /all_status
 * /torumemo
"""

def application(environ, start_response):
    split_result = urlparse.urlsplit(environ['PATH_INFO'])
    paths = filter(None, split_result[2].split('/'))
    headers = [('Content-Type', 'text/plain')]
    status = '200 OK'
    result = ''

    if not paths:
        result = default_message
    else:
        method = getattr(bucho, paths[0], None)
        if method is None:
            status = '404 Not Found'
            result = status
        else:
            result = method()

    start_response(status, headers)
    return [result.encode('utf-8')]


def wsgi_app(environ, start_response):
    # BBB (backword compatibility for bucho-0.0.5
    import warnings
    warnings.warn(
            "The bucho.wsgi.wsgi_app function will be removed in bucho-0.2.0. "
            "Use bucho.wsgi.application instead.", DeprecationWarning)
    return application(environ, start_response)


def app_factory(global_config, **local_conf):
    """ wsgi app factory for Paste """
    return application




import os
import sys
import cmd
import optparse
import bucho
import pkg_resources
from bucho.compat import print_

description = '''
Actions are commands like "show", "latest_status" and "all_status".
If -i is specified or no action is specified on the command line, a
"shell" interpreting actions typed interactively is started. Use the
action "help" to find out about available actions.
'''

bucho_encoding = sys.stdout.encoding
if not bucho_encoding:
    bucho_encoding = 'utf-8'


class BuchoOptions(object):

    def __init__(self):
        programname = os.path.basename(sys.argv[0])
        base, ext = os.path.splitext(programname)
        if ext == ".py":
            if base.endswith('-script'):
                base = base[:-7]
            programname = base
        sys.argv[0] = programname

        usage = 'usage: %prog [options] command'
        parser = optparse.OptionParser(usage=usage, description=description)
        parser.add_option('-i', '--interactive',
                help='start an interactive shell after executing commands',
                dest='interactive',
                action='store_true', default=False)

        parser.add_option('-p', '--prompt',
                help='prompt string',
                dest='prompt',
                action='store', default=(programname+">"))
        parser.add_option('--plugin',
                help='command plugin',
                dest='plugin',
                action='store')
        self.options, self.args = parser.parse_args()


        if not self.args and not self.options.interactive:
            self.options.interactive = 1

    def __getattr__(self, name):
        if hasattr(self.options, name):
            return getattr(self.options, name)
        raise AttributeError(name)

    def __add__(self, obj):
        return obj


class BuchoCmd(cmd.Cmd):

    def __init__(self, options):
        self.options = options
        self.prompt = self.options.prompt + ' '
        if getattr(self.options, 'plugin'):
            cls = self.__class__
            dist = self.options.plugin
            for name, entry_point in pkg_resources.get_entry_map(dist, 'bucho.commands').iteritems():
                setattr(cls, "do_" + name, makeplugincmd(name, entry_point.load()))
                

        cmd.Cmd.__init__(self)

    def do_quit(self, arg):
        return 1

def makeplugincmd(n, func):
    def f(self, arg):
        t = func()
        t = t.encode(bucho_encoding, 'replace')
        t = t.decode(bucho_encoding, 'replace')
        print_(t)
        return 0
    return f
def makecmd(n):
    def f(self, arg):
        t = getattr(bucho, n)()
        t = t.encode(bucho_encoding, 'replace')
        t = t.decode(bucho_encoding, 'replace')
        print_(t)
        return 0
    return f

for name in ['show', 'latest_status', 'all_status', 'torumemo', 'show_gui']:
    setattr(BuchoCmd, "do_" + name, makecmd(name))


def main(options=None, cmdclass=BuchoCmd):
    try:
        if options is None:
            options = BuchoOptions()
        c = cmdclass(options)
    except Exception as e:
        print_(e)
        return

    if options.args:
        c.onecmd(" ".join(options.args))

    if options.interactive:
        try:
            import readline
        except ImportError:
            pass
        c.cmdloop()

def console():
    main()

if __name__ == '__main__':
    main()




from bucho.compat import Tkinter as tk

if tk:
    class BuchoFrame(tk.Frame):
        def __init__(self, text):
            tk.Frame.__init__(self)
            self.text = text
            self.init_view()
    
        def init_view(self):
            self.text_view = tk.Text(self)
            self.text_view.insert(tk.END, self.text)
            self.text_view.configure(state=tk.DISABLED)
            self.text_view.pack(fill=tk.BOTH, expand=True)
    
            
        def run(self):
            self.pack(fill=tk.BOTH, expand=True)
            self.mainloop()



import unittest

class TestIt(unittest.TestCase):
    def _getAUT(self):
        from bucho.wsgi import application
        return application

    def test_show(self):
        app = self._getAUT()
        response = {}
        def start_response(status, headers):
            response['status'] = status
            response['headers'] = headers
            
        environ = {'PATH_INFO': '/show'}
        info = app(environ, start_response)
        self.assertEqual(response['status'], '200 OK')

    def test_not_found(self):
        app = self._getAUT()
        response = {}
        def start_response(status, headers):
            response['status'] = status
            response['headers'] = headers
            
        environ = {'PATH_INFO': '/nothing'}
        info = app(environ, start_response)
        self.assertEqual(response['status'], '404 Not Found')



# -*- coding:utf-8 -*-

try:
    import setuptools
except ImportError:
    import distribute_setup
    distribute_setup.use_setuptools()

import os
import sys

here = os.path.dirname(__file__)
def _read(name):
    try:
        f = open(os.path.join(here, name))
        return f.read()
    except:
        return ""

version = '0.1.2'
name = 'bucho'
short_description = '`bucho` is a package for exercises.'
readme = _read('README.txt')
history = _read('HISTORY.txt')

long_description = readme + "\n" + history

classifiers = [
    'Development Status :: 2 - Pre-Alpha',
    'License :: OSI Approved :: Python Software Foundation License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.2',
    'Topic :: Utilities',
    ]

extra = {}

if sys.version_info >= (3, 0):
    if not getattr(setuptools, '_distribute', False):
        raise RuntimeError(
                'You must installed `distribute` to setup bucho with Python3')
    extra.update(
        use_2to3=True
    )

setuptools.setup(
    name=name,
    version=version,
    description=short_description,
    long_description=long_description,
    classifiers=classifiers,
    keywords=['practice',],
    author='AE35',
    author_email='alpha.echo.35@gmail.com',
    packages = ['bucho'],
    url='http://bitbucket.org/ae35/bucho/',
    license='PSL',
    entry_points = {
        'console_scripts': [
            'bucho=bucho.command:console',
        ],
        'paste.app_factory': [
            'main=bucho.wsgi:app_factory',
        ],
        'bucho.commands': [
            'sample=bucho.plugins:sample',
        ],
    },
    test_suite="bucho",
    **extra
    )




import sys
import types

enable_gui = True
Tkinter = None
try:
    import Tkinter
except ImportError:
    try:
        import tkinter as Tkinter
    except:
        enable_gui = False

PY3 = sys.version_info[0] == 3

if PY3: # pragma: no cover
    string_types = str,
    integer_types = int,
    class_types = type,
    text_type = str
    binary_type = bytes
    long = int
else:
    string_types = basestring,
    integer_types = (int, long)
    class_types = (type, types.ClassType)
    text_type = unicode
    binary_type = str
    long = long

def text_(s, encoding='latin-1', errors='strict'):
    """ If ``s`` is an instance of ``binary_type``, return
    ``s.decode(encoding, errors)``, otherwise return ``s``"""
    if isinstance(s, binary_type):
        return s.decode(encoding, errors)
    return s # pragma: no cover

def bytes_(s, encoding='latin-1', errors='strict'):
    """ If ``s`` is an instance of ``text_type``, return
    ``s.encode(encoding, errors)``, otherwise return ``s``"""
    if isinstance(s, text_type): # pragma: no cover
        return s.encode(encoding, errors)
    return s

def print_(*args):
    x = " ".join(args)
    if x:
        sys.stdout.write(x)
    sys.stdout.write("\n")

